@@ -2,7 +2,7 @@ from requests.auth import AuthBase
 
 
 class AuthBearer(AuthBase):
-    """Bearer Authorization object"""
+    """Bearer authorization object."""
 
     def __init__(self, token):
         self.token = token
